"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Reference = void 0;
const errors = require("./errors");
const Lookup_1 = require("./Lookup");
const ContainerArg_1 = require("./ContainerArg");
class Reference extends ContainerArg_1.ContainerArg {
    constructor(type, lookup) {
        super();
        this.type = type;
        this.lookup = lookup;
        Object.freeze(this);
    }
    getArgument(container) {
        const definitions = this.findDefinitions(container);
        if (Array.isArray(definitions)) {
            return Promise.all(this.findMulti(container).map(d => container.get(d)));
        }
        else {
            return container.get(definitions);
        }
    }
    getDependentServices(container) {
        return this.findDefinitions(container);
    }
    findDefinitions(container) {
        if (this.type === 'one') {
            return this.findOne(container);
        }
        return this.findMulti(container);
    }
    findOne(container) {
        const definitions = this.lookup.find(container);
        if (Array.isArray(definitions)) {
            if (definitions.length === 0) {
                throw errors.NO_MATCHING_SERVICE('No matching service for following lookup: ' + this.lookup);
            }
            if (definitions.length > 1) {
                const servicesNames = definitions.map(s => s.name).join(', ');
                throw errors.AMBIGUOUS_SERVICE(`Multiple services found (${servicesNames}) with following lookup: ${this.lookup}`);
            }
            return definitions[0];
        }
        else {
            return definitions;
        }
    }
    findMulti(container) {
        return this.lookup.find(container);
    }
}
exports.Reference = Reference;
Reference.one = {
    name(name) {
        return new Reference('one', new Lookup_1.Lookup.ByServiceName(name));
    },
    predicate(predicate) {
        return new Reference('one', new Lookup_1.Lookup.ByPredicate(predicate));
    },
    annotation(predicate) {
        return new Reference('one', new Lookup_1.Lookup.ByAnnotation(predicate));
    },
    type(type) {
        return new Reference('one', new Lookup_1.Lookup.ByType(type));
    }
};
Reference.multi = {
    predicate(predicate) {
        return new Reference('multi', new Lookup_1.Lookup.ByPredicate(predicate));
    },
    annotation(predicate) {
        return new Reference('multi', new Lookup_1.Lookup.ByAnnotation(predicate));
    },
    type(type) {
        return new Reference('multi', new Lookup_1.Lookup.ByType(type));
    }
};
