"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.namespaceEntry = exports.createNamespace = void 0;
const is = require("predicates");
function createNamespace(object) {
    return createProxyForPrefix([], object);
}
exports.createNamespace = createNamespace;
exports.namespaceEntry = Symbol('namespaceEntry');
function createProxyForPrefix(prefixParts, object) {
    return new Proxy(object, {
        get(target, key) {
            if (!(key in target)) {
                throw new Error(`There is no name for path: ${getKey(prefixParts, key)}`);
            }
            const value = target[key];
            if (is.plainObject(value)) {
                return createProxyForPrefix(prefixParts.concat([key]), value);
            }
            else if (is.string(value)) {
                return getKey(prefixParts, value);
            }
            else if (value === exports.namespaceEntry) {
                return getKey(prefixParts, key);
            }
            return value;
        }
    });
}
function getKey(prefixParts, lastPart) {
    return prefixParts.concat([lastPart]).join('.');
}
