"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AutowiredService = void 0;
const Service_1 = require("./Service");
const serviceMetadata_1 = require("../serviceMetadata");
const is = require("predicates");
const TypeRef_1 = require("../TypeRef");
const index_1 = require("../index");
const util_1 = require("util");
function AutowiredService(name) {
    return (clazz) => {
        const paramTypes = Reflect.getMetadata("design:paramtypes", clazz);
        const metadata = serviceMetadata_1.ensureMetadata(clazz);
        if (is.array(paramTypes) && clazz.length > 0) {
            for (const [index, paramType] of paramTypes.entries()) {
                if (metadata.constructorArguments[index]) {
                    continue;
                }
                const ref = TypeRef_1.TypeRef.createFromType(paramType);
                if (ref === undefined) {
                    throw index_1.errors.AUTOWIRING_FAILED(util_1.format(index_1.errors.AUTOWIRING_FAILED.defaultMessage, `constructor (of ${clazz.name}) argument nr: ${index}`));
                }
                metadata.constructorArguments[index] = index_1.Reference.one.type(ref);
            }
        }
        else if (clazz.length > 0) {
            throw index_1.errors.AUTOWIRING_NO_METADATA();
        }
        Service_1.Service(name)(clazz);
    };
}
exports.AutowiredService = AutowiredService;
