"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Inject = void 0;
const ContainerArg_1 = require("../ContainerArg");
require("reflect-metadata");
const Reference_1 = require("../Reference");
const is = require("predicates");
const serviceMetadata_1 = require("../serviceMetadata");
const types_1 = require("../types");
const TypeRef_1 = require("../TypeRef");
const index_1 = require("../index");
const util_1 = require("util");
const assertServiceNameOrContainerArg = is.assert(is.any(types_1.ServiceName.is, ContainerArg_1.ContainerArg.is, TypeRef_1.TypeRef.is, is.func, is.undefined), '@Inject argument must be a string that represents service name, symbol, function or instance of ContainerArg or TypeRef');
function Inject(ref) {
    assertServiceNameOrContainerArg(ref);
    return function (target, property, indexOrDescriptor) {
        const isParameterDecorator = typeof indexOrDescriptor === 'number';
        let arg;
        if (ref === undefined) {
            if (isParameterDecorator) {
                throw new Error('Using @Inject decorator without ref for function arguments is prohibited');
            }
            else {
                const designType = Reflect.getMetadata("design:type", target, property);
                arg = createReferenceForType(designType, `property ${property.toString()}`);
            }
        }
        else if (TypeRef_1.TypeRef.is(ref)) {
            arg = Reference_1.Reference.one.type(ref);
        }
        else if (types_1.ServiceName.is(ref)) {
            arg = Reference_1.Reference.one.name(ref);
        }
        else if (ContainerArg_1.ContainerArg.is(ref)) {
            arg = ref;
        }
        else {
            arg = createReferenceForType(ref, isParameterDecorator ? `constructor (of ${target.name}) argument nr: ${indexOrDescriptor}` : `property ${property.toString()}`);
        }
        if (isParameterDecorator) {
            serviceMetadata_1.ensureMetadata(target).constructorArguments[indexOrDescriptor] = arg;
        }
        else {
            serviceMetadata_1.ensureMetadata(target.constructor).propertiesInjectors.set(property, arg);
        }
    };
}
exports.Inject = Inject;
function createReferenceForType(func, ...args) {
    const type = TypeRef_1.TypeRef.createFromType(func);
    if (type === undefined) {
        throw index_1.errors.AUTOWIRING_FAILED(util_1.format(index_1.errors.AUTOWIRING_FAILED.defaultMessage, ...args));
    }
    return Reference_1.Reference.one.type(type);
}
